package com.biganto.visual.roompark.domain.use_case

//import io.reactivex.Observable
import com.biganto.visual.roompark.data.repository.db.requrey.model.SubscriptionEntity
import com.biganto.visual.roompark.domain.contract.AuthContract
import com.biganto.visual.roompark.domain.contract.DeviceUtilsContract
import com.biganto.visual.roompark.domain.contract.SubscriptionContract
import com.biganto.visual.roompark.domain.model.SubscriptionModel
import com.biganto.visual.roompark.domain.model.SubscriptionTopic
import com.biganto.visual.roompark.domain.model.fromEntity
import io.reactivex.Observable
import io.reactivex.rxkotlin.Observables
import javax.inject.Inject
//import io.reactivex.rxkotlin

/**
 * Created by Vladislav Bogdashkin on 01.11.2019.
 */

class SubscriptionUseCase @Inject constructor(
    private val subscription: SubscriptionContract,
    private val utils: DeviceUtilsContract,
    private val auth: AuthContract
) {
    fun subscribeTopic(subId: Int, topic: SubscriptionTopic) =
        Observables.zip(auth.currentUser(), utils.getDeviceId()) { user, token ->
            subscription.subscribeTopic(
                user
                , subId
                , token
                , topic = topic.topicName,
                topic_id = topic.topicId
            )
        }

    fun unSubscribeTopic(subId: Int, topic: SubscriptionTopic) =
        Observables.zip(auth.currentUser(), utils.getDeviceId()) { user, token ->
            subscription.unSubscribeTopic(
                user
                , subId
                , token
                , topic = topic.topicName,
                topic_id = topic.topicId
            )
        }

    fun getSubscriptions(topic: SubscriptionTopic): Observable<SubscriptionModel> =
        auth.currentUser()
            .map {
                var sub = it.subscriptions
                    ?.firstOrNull { it.topic == topic.topicName && it.number == topic.topicId }
                if (sub == null) {
                    sub = SubscriptionEntity()
                    sub.setOwner(it)
                    sub.setTopic(topic.topicName)
                    sub.setNumber(topic.topicId)
                    sub.setState(false)
                }
                subscription.saveSubscribeState(sub as SubscriptionEntity).blockingFirst()
            }
            .map(::fromEntity)
}





